//////////////////////////////////////////////////////////////////////
// geometry.cpp: implementation for the Geometry class.
// -Kyoung Shin Park (2014 Spring)
//////////////////////////////////////////////////////////////////////
#include "moglclass.h"
#include "geometryPositionColor.h"

namespace moglclass
{

GeometryPositionColor::GeometryPositionColor()
{
	color[0] = randomVal(0.0f, 1.0f); color[1] = randomVal(0.0f, 1.0f); color[2] = randomVal(0.0f, 1.0f);
	numVertices = 0;
	isLoaded = false;

	// animation
	start = glm::vec3(0, 0, 0);
	end = glm::vec3(1, 0, 0);
	duration = 1000;
	reset();
	curve = NULL;
}

void GeometryPositionColor::setAnimation(glm::vec3 start_, glm::vec3 end_, float duration_)
{
	start = start_; 
	end = end_; 
	duration = duration_; 
	p = start; 
	init();
}

void GeometryPositionColor::setKeyframeAnimation(std::vector<KeyFrame> frames_)
{
	curve = new CatmullRomCurveAnimation(frames_, true);
	init();
}

bool GeometryPositionColor::update(float elapsedTime)
{
	if (active)
	{
		float fraction = elapsedTime/duration;
		p = glm::mix(start, end, fraction); 	//p = start + (end - start) * elapsedTime/duration;
		printf("pos=%f %f %f\n ", p[0], p[1], p[2]);

		// if time is greater than duration, then set time to duration and set active to false
		if (elapsedTime >= duration) 
		{
			done = true;
			active = false;
		}

		init();
	}
	return true;
}

Triangle::Triangle(glm::vec3 p_, glm::vec3 v1_, glm::vec3 v2_, glm::vec3 v3_) : GeometryPositionColor()
{
	set(p_, v1_, v2_, v3_);
}

void Triangle::set(glm::vec3 p_, glm::vec3 v1_, glm::vec3 v2_, glm::vec3 v3_)
{
	p = p_;
	v1 = v1_;
	v2 = v2_;
	v3 = v3_;
	init();
}

void Triangle::init()
{
	glm::vec3 t1 = p + v1;
	glm::vec3 t2 = p + v2;
	glm::vec3 t3 = p + v3;

	// face 1
	vbo.addData(&t1[0], sizeof(glm::vec3)); // vertex position
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&t2[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3));
	vbo.addData(&t3[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3));

	numVertices = 3;

	// VAO & VBOs
	vbo.createVBO();
	vbo.bindVBO();
	vbo.uploadDataToGPU(GL_STATIC_DRAW);

	glGenVertexArrays(1, &vao); 
	glBindVertexArray(vao);

	int iDataStride = 2 * sizeof(glm::vec3); // vertex & color only
	int iDataOffset = 0;
	glEnableVertexAttribArray(0);
	glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
	iDataOffset += sizeof(glm::vec3);
	glEnableVertexAttribArray(1);
	glVertexAttribPointer(1, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);

	isLoaded = true;
}

void Triangle::draw(bool wireframe)
{
	if (!isLoaded) return;
	glBindVertexArray(vao);
	if (wireframe)
		glDrawArrays(GL_LINE_LOOP, 0, numVertices);
	else
		glDrawArrays(GL_TRIANGLES, 0, numVertices);
}


Circle::Circle(glm::vec3 p_, float radius_, int slices_) : GeometryPositionColor()
{
	set(p_, radius_, slices_);
}

void Circle::set(glm::vec3 p_, float radius_, int slices_)
{
	p = p_; // center position
	radius = radius_;
	slices = slices_;
	init();
}

void Circle::init()
{
	glm::vec3 vertex;
	float theta = (float) (2*M_PI/slices);
	for (int i=0; i<slices; i++) 
	{
		vertex[0] = p[0] + radius * cos(theta * i);
		vertex[1] = p[1] + radius * sin(theta * i);
		vertex[2] = p[2];
		vbo.addData(&vertex[0], sizeof(glm::vec3)); // vertex position
		vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	}
	numVertices = slices;

	// VAO & VBOs
	vbo.createVBO();
	vbo.bindVBO();
	vbo.uploadDataToGPU(GL_STATIC_DRAW);

	glGenVertexArrays(1, &vao); 
	glBindVertexArray(vao);

	int iDataStride = 2 * sizeof(glm::vec3); // vertex & color only
	int iDataOffset = 0;
	glEnableVertexAttribArray(0);
	glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
	iDataOffset += sizeof(glm::vec3);
	glEnableVertexAttribArray(1);
	glVertexAttribPointer(1, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);

	isLoaded = true;
}

void Circle::draw(bool wireframe)
{
	if (!isLoaded) return;
	glBindVertexArray(vao);
	if (wireframe)
		glDrawArrays(GL_LINE_LOOP, 0, numVertices);	// draw wireframe circle  
	else
		glDrawArrays(GL_TRIANGLE_FAN, 0, numVertices);	// draw solid circle  
}

Quad::Quad(glm::vec3 p_, glm::vec3 n_, glm::vec3 u_, float width_, float height_) : GeometryPositionColor()
{
	set(p_, n_, u_, width_, height_);
}

void Quad::set(glm::vec3 p_, glm::vec3 n_, glm::vec3 u_, float width_, float height_)
{
	p = p_; // center position
	n = n_; // normal vector
	u = u_; // up vector
	width = width_;
	height = height_;
	init();
}

void Quad::init()
{
	glm::vec3 left = glm::cross(n, u);
	glm::vec3 uppercenter = (u * height/2.0f) + p;
	glm::vec3 tl = uppercenter + (left * width/2.0f);
	glm::vec3 tr = uppercenter - (left * width/2.0f);
	glm::vec3 bl = tl - (u * height);
	glm::vec3 br = tr - (u * height);

	// face 1
	vbo.addData(&bl[0], sizeof(glm::vec3)); // vertex position
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&br[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3));
	vbo.addData(&tr[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3));

	// face 2
	vbo.addData(&bl[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3));
	vbo.addData(&tr[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3));
	vbo.addData(&tl[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3));

	numVertices = 6;

	// VAO & VBOs
	vbo.createVBO();
	vbo.bindVBO();
	vbo.uploadDataToGPU(GL_STATIC_DRAW);

	glGenVertexArrays(1, &vao); 
	glBindVertexArray(vao);

	int iDataStride = 2 * sizeof(glm::vec3); // vertex & color only
	int iDataOffset = 0;
	glEnableVertexAttribArray(0);
	glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
	iDataOffset += sizeof(glm::vec3);
	glEnableVertexAttribArray(1);
	glVertexAttribPointer(1, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);

	isLoaded = true;
}

void Quad::draw(bool wireframe)
{
	if (!isLoaded) return;
	glBindVertexArray(vao);
	if (wireframe)
		glDrawArrays(GL_LINE_LOOP, 0, numVertices);
	else
		glDrawArrays(GL_TRIANGLES, 0, numVertices);
}

Sphere::Sphere(glm::vec3 p_, float radius_, int slices_, int stacks_) : GeometryPositionColor()
{
	set(p_, radius_, slices_, stacks_);
}

void Sphere::set(glm::vec3 p_, float radius_, int slices_, int stacks_)
{
	p = p_; // center position
	radius = radius_;
	slices = slices_;
	stacks = stacks_;
	init();
}

void Sphere::init()
{
	numVertices = 0;
	glm::vec3 vertex;
	float lonstep = (float) (M_PI/stacks);
	float latstep = (float) (M_PI/slices);
	float lon, lat;

	for (lon = 0.0f; lon <= 2*M_PI; lon += (lonstep))
	{
		for (lat = 0.0; lat <= M_PI + latstep; lat += (latstep))
		{
			vertex[0] = p[0] + radius * cosf(lon)*sinf(lat);
			vertex[1] = p[1] + radius * sinf(lon)*sinf(lat);
			vertex[2] = p[2] + radius * cosf(lat);
			vbo.addData(&vertex[0], sizeof(glm::vec3)); // vertex position
			vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
			numVertices++;

			vertex[0] = p[0] + radius * cosf(lon + lonstep)*sinf(lat);
			vertex[1] = p[1] + radius * sinf(lon + lonstep)*sinf(lat);
			vertex[2] = p[2] + radius * cosf(lat);
			vbo.addData(&vertex[0], sizeof(glm::vec3)); // vertex position
			vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
			numVertices++;
		}
	}

	// VAO & VBOs
	vbo.createVBO();
	vbo.bindVBO();
	vbo.uploadDataToGPU(GL_STATIC_DRAW);

	glGenVertexArrays(1, &vao); 
	glBindVertexArray(vao);

	int iDataStride = 2 * sizeof(glm::vec3); // vertex & color
	int iDataOffset = 0;
	glEnableVertexAttribArray(0);
	glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
	iDataOffset += sizeof(glm::vec3);
	glEnableVertexAttribArray(1);
	glVertexAttribPointer(1, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);

	isLoaded = true;
}

void Sphere::draw(bool wireframe)
{
	if (!isLoaded) return;
	glBindVertexArray(vao);
	if (wireframe)
		glDrawArrays(GL_LINE_STRIP, 0, numVertices);
	else
		glDrawArrays(GL_TRIANGLE_STRIP, 0, numVertices);	// draw sphere  
}

Cylinder::Cylinder(glm::vec3 p_, float radius_, float height_, int slices_) : GeometryPositionColor()
{
	set(p_, radius_, height_, slices_);
}

void Cylinder::set(glm::vec3 p_, float radius_, float height_, int slices_)
{
	p = p_;
	radius = radius_;
	height = height_;
	slices = slices_;
	init();
}

void Cylinder::init()
{
	numVertices = 0;
	glm::vec3 vertex;

	float theta = (float) (2*M_PI/slices);
	for (int i = 0; i <= slices; i++) 
	{
		vertex[0] = p[0] + radius * cosf(theta * i);
		vertex[1] = p[1] + -height/2.0f;
		vertex[2] = p[2] + radius * sinf(theta * i);
		vbo.addData(&vertex[0], sizeof(glm::vec3)); // vertex position
		vertex[1] = 0.0f;
		vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
		numVertices++;

		vertex[0] = p[0] + radius * cosf(theta * i);
		vertex[1] = p[1] + height/2.0f;
		vertex[2] = p[2] + radius * sinf(theta * i);
		vbo.addData(&vertex[0], sizeof(glm::vec3)); // vertex position
		vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
		numVertices++;
	}

	// VAO & VBOs
	vbo.createVBO();
	vbo.bindVBO();
	vbo.uploadDataToGPU(GL_STATIC_DRAW);

	glGenVertexArrays(1, &vao); 
	glBindVertexArray(vao);

	int iDataStride = 2 * sizeof(glm::vec3); // vertex & color
	int iDataOffset = 0;
	glEnableVertexAttribArray(0);
	glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
	iDataOffset += sizeof(glm::vec3);
	glEnableVertexAttribArray(1);
	glVertexAttribPointer(1, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
	
	isLoaded = true;
}

void Cylinder::draw(bool wireframe)
{
	if (!isLoaded) return;
	glBindVertexArray(vao);
	if (wireframe)
		glDrawArrays(GL_LINE_STRIP, 0, numVertices);
	else
		glDrawArrays(GL_TRIANGLE_STRIP, 0, numVertices);	// draw cylinder  
}

Cube::Cube(glm::vec3 p_, float size_) : GeometryPositionColor()
{
	set(p_, size_);
}

void Cube::set(glm::vec3 p_, float size_)
{
	p = p_;
	size = (float) (size_ * 0.5);
	init();
}

void Cube::init()
{
	glm::vec3 fll = p + glm::vec3(-size, -size,  size);
	glm::vec3 flr = p + glm::vec3( size, -size,  size);
	glm::vec3 fur = p + glm::vec3( size,  size,  size);
	glm::vec3 ful = p + glm::vec3(-size,  size,  size);
	glm::vec3 bll = p + glm::vec3(-size, -size,  -size);
	glm::vec3 blr = p + glm::vec3( size, -size,  -size);
	glm::vec3 bur = p + glm::vec3( size,  size,  -size);
	glm::vec3 bul = p + glm::vec3(-size,  size,  -size);

	// Front face
	vbo.addData(&fll[0], sizeof(glm::vec3)); // vertex position
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&flr[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); 
	vbo.addData(&fur[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); 
	vbo.addData(&fll[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); 
	vbo.addData(&fur[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); 
	vbo.addData(&ful[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); 

	// Back face
	vbo.addData(&blr[0], sizeof(glm::vec3)); // vertex position
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&bll[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3));
	vbo.addData(&bul[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3));
	vbo.addData(&bll[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3));
	vbo.addData(&bul[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3));
	vbo.addData(&bur[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3));

	// Left face
	vbo.addData(&bll[0], sizeof(glm::vec3)); // vertex position
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color 
	vbo.addData(&fll[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); 
	vbo.addData(&ful[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); 
	vbo.addData(&bll[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); 
	vbo.addData(&ful[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); 
	vbo.addData(&bul[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); 

	// Right face
	vbo.addData(&flr[0], sizeof(glm::vec3)); // vertex position
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&blr[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); 
	vbo.addData(&bur[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); 
	vbo.addData(&flr[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); 
	vbo.addData(&bur[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); 
	vbo.addData(&fur[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); 

	// Bottom face
	vbo.addData(&bll[0], sizeof(glm::vec3)); // vertex position
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&blr[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); 
	vbo.addData(&flr[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); 
	vbo.addData(&bll[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); 
	vbo.addData(&flr[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); 
	vbo.addData(&fll[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); 

	// Top face
	vbo.addData(&ful[0], sizeof(glm::vec3)); // vertex position
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&fur[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3));
	vbo.addData(&bur[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3));
	vbo.addData(&ful[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3));
	vbo.addData(&bur[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3));
	vbo.addData(&bul[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3));

	numVertices = 36;

	// VAO & VBOs
	vbo.createVBO();
	vbo.bindVBO();
	vbo.uploadDataToGPU(GL_STATIC_DRAW);

	glGenVertexArrays(1, &vao); 
	glBindVertexArray(vao);

	int iDataStride = 2 * sizeof(glm::vec3); // vertex & color
	int iDataOffset = 0;
	glEnableVertexAttribArray(0);
	glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
	iDataOffset += sizeof(glm::vec3);
	glEnableVertexAttribArray(1);
	glVertexAttribPointer(1, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);

	isLoaded = true;
}

void Cube::draw(bool wireframe)
{
	if (!isLoaded) return;
	glBindVertexArray(vao);
	if (wireframe)
		glDrawArrays(GL_LINE_LOOP, 0, numVertices);
	else
		glDrawArrays(GL_TRIANGLES, 0, numVertices);
}

Torus::Torus(glm::vec3 p_, float radius_, float tubeRadius_, int rings_, int sides_) : GeometryPositionColor()
{
	set(p_, radius_, tubeRadius_, rings_, sides_);
}

void Torus::set(glm::vec3 p_, float radius_, float tubeRadius_, int rings_, int sides_)
{
	p = p_;
	radius = radius_;			// radius of torus
	tubeRadius = tubeRadius_;	// radius of tube
	rings = rings_;				// number of radial divisions for torus
	sides = sides_;				// number of tube subdivisions
	init();
}

void Torus::init()
{
	numVertices = 0;
	glm::vec3 vertex;	

   	float theta = (float) (2*M_PI/rings);	// torus outer radius
	float phi = (float) (2*M_PI/sides);		// tube inner radius
	float u, v;

	for (v = 0.0f; v < 2*M_PI; v += phi) 
	{
		for (u = 0.0f; u < 2*M_PI; u += theta) 
		{
			vertex[0] = p[0] + (radius + tubeRadius * cos(v)) * cos(u);
			vertex[1] = p[1] + (radius + tubeRadius * cos(v)) * sin(u);
			vertex[2] = p[2] + tubeRadius * sin(v);
			vbo.addData(&vertex[0], sizeof(glm::vec3)); // vertex position 0
			vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color

			vertex[0] = p[0] + (radius + tubeRadius * cos(v)) * cos(u + theta);
			vertex[1] = p[1] + (radius + tubeRadius * cos(v)) * sin(u + theta);
			vertex[2] = p[2] + tubeRadius * sin(v);
			vbo.addData(&vertex[0], sizeof(glm::vec3)); // vertex position 1
			vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color

			vertex[0] = p[0] + (radius + tubeRadius * cos(v + phi)) * cos(u + theta);
			vertex[1] = p[1] + (radius + tubeRadius * cos(v + phi)) * sin(u + theta);
			vertex[2] = p[2] + tubeRadius * sin(v + phi);
			vbo.addData(&vertex[0], sizeof(glm::vec3)); // vertex position 2
			vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color

			vertex[0] = p[0] + (radius + tubeRadius * cos(v + phi)) * cos(u + theta);
			vertex[1] = p[1] + (radius + tubeRadius * cos(v + phi)) * sin(u + theta);
			vertex[2] = p[2] + tubeRadius * sin(v + phi);
			vbo.addData(&vertex[0], sizeof(glm::vec3)); // vertex position 2
			vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color

			vertex[0] = p[0] + (radius + tubeRadius * cos(v + phi)) * cos(u);
			vertex[1] = p[1] + (radius + tubeRadius * cos(v + phi)) * sin(u);
			vertex[2] = p[2] + tubeRadius * sin(v + phi);
			vbo.addData(&vertex[0], sizeof(glm::vec3)); // vertex position 3
			vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color

			vertex[0] = p[0] + (radius + tubeRadius * cos(v)) * cos(u);
			vertex[1] = p[1] + (radius + tubeRadius * cos(v)) * sin(u);
			vertex[2] = p[2] + tubeRadius * sin(v);
			vbo.addData(&vertex[0], sizeof(glm::vec3)); // vertex position 0
			vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color

			numVertices += 6;
		}
	}
	//fprintf(stderr, "Torus numVertices=%d\n", numVertices);

	// VAO & VBOs
	vbo.createVBO();
	vbo.bindVBO();
	vbo.uploadDataToGPU(GL_STATIC_DRAW);

	glGenVertexArrays(1, &vao); 
	glBindVertexArray(vao);

	int iDataStride = 2 * sizeof(glm::vec3); // vertex & color
	int iDataOffset = 0;
	glEnableVertexAttribArray(0);
	glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
	iDataOffset += sizeof(glm::vec3);
	glEnableVertexAttribArray(1);
	glVertexAttribPointer(1, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);

	isLoaded = true;
}

void Torus::draw(bool wireframe)
{
	if (!isLoaded) return;
	glBindVertexArray(vao);
	if (wireframe)
		glDrawArrays(GL_LINE_LOOP, 0, numVertices);
	else
		glDrawArrays(GL_TRIANGLES, 0, numVertices);
}

Parallelepiped::Parallelepiped(glm::vec3 p_, glm::vec3 u_, glm::vec3 v_, glm::vec3 w_) : GeometryPositionColor()
{
	set(p_, u_, v_, w_);
}

void Parallelepiped::set(glm::vec3 p_, glm::vec3 u_, glm::vec3 v_, glm::vec3 w_)
{
	p = p_;
	u = u_;
	v = v_;
	w = w_;
	init();
}

void Parallelepiped::init()
{
	glm::vec3 pu = p + u;
	glm::vec3 pv = p + v;
	glm::vec3 pw = p + w;
	glm::vec3 puv = p + u + v;
	glm::vec3 pvw = p + v + w;
	glm::vec3 puw = p + u + w;
	glm::vec3 puvw = p + u + v + w;

	glm::vec3 frontNormal = glm::cross(u, w);
	glm::vec3 backNormal = glm::cross(w, u);
	glm::vec3 rightNormal = glm::cross(v, w);
	glm::vec3 leftNormal = glm::cross(w, v);
	glm::vec3 topNormal = glm::cross(u, v);
	glm::vec3 bottomNormal = glm::cross(v, u);

	// Front face
	vbo.addData(&p[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&pu[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&puw[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&p[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&puw[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&pw[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color

	// Back face
	vbo.addData(&puv[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&pv[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&pvw[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&puv[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&pvw[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&puvw[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color

	// Left face
	vbo.addData(&pv[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&p[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&pw[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&pv[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&pw[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&pvw[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color

	// Right face
	vbo.addData(&pu[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&puv[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&puvw[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&pu[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&puvw[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&puw[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color

	// Top face
	vbo.addData(&pw[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&puw[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&puvw[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&pw[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&puvw[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&pvw[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color

	// Bottom face
	vbo.addData(&pv[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&puv[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&pu[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&pv[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&pu[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color
	vbo.addData(&p[0], sizeof(glm::vec3));
	vbo.addData(&color[0], sizeof(glm::vec3)); // vertex color

	numVertices = 36;

	// VAO & VBOs
	vbo.createVBO();
	vbo.bindVBO();
	vbo.uploadDataToGPU(GL_STATIC_DRAW);

	glGenVertexArrays(1, &vao); 
	glBindVertexArray(vao);

	int iDataStride = 2 * sizeof(glm::vec3); // vertex & color
	int iDataOffset = 0;
	glEnableVertexAttribArray(0);
	glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);
	iDataOffset += sizeof(glm::vec3);
	glEnableVertexAttribArray(1);
	glVertexAttribPointer(1, 3, GL_FLOAT, GL_FALSE, iDataStride, (void*)iDataOffset);

	isLoaded = true;
}

void Parallelepiped::draw(bool wireframe)
{
	if (!isLoaded) return;
	glBindVertexArray(vao);
	glDrawArrays(GL_TRIANGLES, 0, numVertices);
}

Sa::Sa(glm::vec3 p_) : GeometryPositionColor()
{
	p = p_;
	for (int i=0; i<4; i++) pipe[i] = Parallelepiped();
	init();

	// keyframe animation
	std::vector<KeyFrame> keyframes;
	keyframes.push_back(KeyFrame(glm::vec3(0, 0, 0), 0));
	keyframes.push_back(KeyFrame(glm::vec3(1, 1, 0), 2000));
	keyframes.push_back(KeyFrame(glm::vec3(-1, 2, 0), 4000));
	keyframes.push_back(KeyFrame(glm::vec3(2, 3, 0), 6000));
	setKeyframeAnimation(keyframes);
}

void Sa::init() // ""
{
	glm::vec3 p0 = p;
	pipe[0].set(p0, glm::vec3(0.3f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 0.3f), glm::vec3(0.7f, 1.0f, 0.0f));
	glm::vec3 p1 = p + glm::vec3(1.4f, 0.0f, 0.0f);
	pipe[1].set(p1, glm::vec3(0.3f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 0.3f), glm::vec3(-0.7f, 1.0f, 0.0f));
	glm::vec3 p2 = p + glm::vec3(2.0f, 0.0f, 0.0f);
	pipe[2].set(p2, glm::vec3(0.3f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 0.3f), glm::vec3(0.0f, 1.0f, 0.0f));
	glm::vec3 p3 = p + glm::vec3(2.3f, 0.35f, 0.0f);
	pipe[3].set(p3, glm::vec3(0.5f, 0.0f, 0.0f), glm::vec3(0.0f, 0.0f, 0.3f), glm::vec3(0.0f, 0.3f, 0.0f));
}

void Sa::draw(bool wireframe)
{
	for (int i=0; i<4; i++)	pipe[i].draw();
}

bool Sa::update(float elapsedTime)
{
	if (active)
	{
		if (curve)
		{
			curve->updatePosition(elapsedTime);
			p = curve->getPosition();

			// if time is greater than duration, then set time to duration and set active to false
			if (!curve->getLoop() && elapsedTime >=  curve->getDuration())
			{
				done = true;
				active = false;
			}
		}
		init();
	}
	return true;
}

} // namespace